#include "bezier.h"

#include <GL/glew.h>
#include <GL/freeglut.h>
#include <iostream>
#include <cstdlib>

Bezier::Bezier(void){
    max_points_=4;
    num_points_=0;
}

Bezier::~Bezier(){
    glDisableVertexAttribArray(0);
    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glDeleteBuffers(1, &vertex_buffer_);

    glBindVertexArray(0);
    glDeleteVertexArrays(1,&vao_);

}

void Bezier::Initialize(){
    glGenVertexArrays(1, &vao_);
    glBindVertexArray(vao_);

    glGenBuffers(1, &vertex_buffer_);
    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer_);
    glBufferData(GL_ARRAY_BUFFER, max_points_*2*sizeof(float), NULL, GL_DYNAMIC_DRAW);
    glVertexAttribPointer(0, 2, GL_FLOAT, GL_FALSE, 0, (GLvoid*)0);
    glEnableVertexAttribArray(0);

    glBindVertexArray(0);
}



void Bezier::Draw(const Program &program, const Program &bezier_program){

    glBindVertexArray(vao_);
    glUseProgram(program);

    glDrawArrays(GL_LINE_STRIP, 0, num_points_);
    glDrawArrays(GL_POINTS, 0, num_points_);

    if(num_points_ == max_points_){
        glUseProgram(bezier_program);
        glDrawArrays(GL_LINES_ADJACENCY, 0, num_points_);
    }

    glBindVertexArray(0);
    glUseProgram(0);
}

bool Bezier::AddPoint(float x_coord, float y_coord){
    if(num_points_ == max_points_) return false;
    float vertex[2];
    vertex[0]=x_coord;
    vertex[1]=y_coord;
    glBindVertexArray(vao_);
    glBindBuffer(GL_ARRAY_BUFFER, vertex_buffer_);
    glBufferSubData(GL_ARRAY_BUFFER, (num_points_++)*2*sizeof(float), 2*sizeof(float), vertex);
    glBindVertexArray(0);
    return true;
}

